@@ -44,13 +44,17 @@ def getCap(result, limit):
         suffix = ''
     return cutCaption(result.cap, suffix, limit)
 
-def sendVideo(chat, result):
-    os.system('mkdir tmp > /dev/null 2>&1')
-    with open('tmp/video.mp4', 'wb') as f:
-        f.write(cached_url.get(result.video, force_cache=True, mode='b'))
-    if os.stat('tmp/video.mp4').st_size > 50 * 1024 * 1024:
-        return []
-    group = [InputMediaVideo(open('tmp/video.mp4', 'rb'), 
+def sendVideo(chat, result, use_url = False):
+    if use_url:
+        video_source = result.video
+    else:
+        os.system('mkdir tmp > /dev/null 2>&1')
+        with open('tmp/video.mp4', 'wb') as f:
+            f.write(cached_url.get(result.video, force_cache=True, mode='b'))
+        if os.stat('tmp/video.mp4').st_size > 50 * 1024 * 1024:
+            return []
+        video_source = open('tmp/video.mp4', 'rb')
+    group = [InputMediaVideo(video_source, 
         caption=getCap(result, 1000), parse_mode=result.getParseMode())]
     return chat.bot.send_media_group(chat.id, group, timeout = 20*60)
 
@@ -86,14 +90,20 @@ def prepareCache(img):
     if '.sinaimg.cn' in img:
         cached_url.get(img, mode='b', headers={'referer': 'https://m.weibo.cn/'}, force_cache = True)
         
-def send_v2_imp(chat, result, rotate=0, send_all=False, no_cut=False, size_factor = None, rotate_detail = {}, start_page = 0):
-    # todo: cached_url may want to make sure the video fn ends with mp4...  
+def send_v2_imp(chat, result, rotate=0, send_all=False, no_cut=False, size_factor = None, rotate_detail = {}, start_page = 0, use_url = False):
+    # may need to revisit this line
+    if (not result.video) and len(result.imgs) == 1 and result.imgs[0].endswith('mp4'):
+        result.video = result.imgs[0]
     if result.video:
         try:
-            return sendVideo(chat, result)
+            send_result = sendVideo(chat, result, use_url = use_url)
+            if send_result:
+                return send_result
+            raise Exception('video send failed')
         except Exception as e:
-            if not result.imgs:
-                raise e
+            if use_url:
+                raise Exception('try download video next time')
+            raise e
 
     img_limit = start_page * 10 + (100 if send_all else 10)
     [prepareCache(img) for img in result.imgs]
@@ -125,13 +135,32 @@ def send_v2_imp(chat, result, rotate=0, send_all=False, no_cut=False, size_facto
             disable_web_page_preview = (not isUrl(result.cap)))]
 
 def send_v2(chat, result, rotate=0, send_all=False, no_cut=False, size_factor = None, rotate_detail = {}, start_page=0):
-    try:
-        return send_v2_imp(chat, result, rotate=rotate, send_all=send_all, no_cut=no_cut, size_factor = size_factor, rotate_detail = rotate_detail, start_page=start_page)
-    except Exception as e:
-        if "Can't parse entities" in str(e):
-            result.no_parse = True
-            return send_v2_imp(chat, result, rotate=rotate, send_all=send_all, no_cut=no_cut, size_factor = size_factor, rotate_detail = rotate_detail, start_page=start_page)
-        raise e
+    args = {'rotate': rotate, 'send_all': send_all, 'no_cut': no_cut, 
+      'size_factor': size_factor, 'rotate_detail': rotate_detail, 
+      'start_page': start_page, 'use_url': True}
+    encoutered_errors = set()
+    while True:
+        try:
+            return send_v2_imp(chat, result, **args)
+        except Exception as e:
+            if str(e) in encoutered_errors:
+                raise e
+            if len(encoutered_errors) > 5:
+                raise e
+            encoutered_errors.add(str(e))
+            if 'Timed out' in str(e):
+                return []
+            if str(e).endswith('seconds'):
+                wait = int(str(e).split()[-2])
+                time.sleep(wait + 5)
+                continue
+            if "Can't parse entities" in str(e):
+                result.no_parse = True
+                continue
+            if 'try download video next time' in str(e):
+                args['use_url'] = False
+                continue
+            raise e
 
 def send(chat, url, result, rotate=0, send_all=False):
     result.url = url
